"""QtPie decorators."""

from qtpie.decorators.action import action
from qtpie.decorators.menu import menu
from qtpie.decorators.slot import slot
from qtpie.decorators.widget import widget
from qtpie.decorators.window import window

__all__ = ["action", "menu", "slot", "widget", "window"]
